﻿namespace Multicad.Samples.Tables
{
  using System;
  using System.Collections.Generic;
  using System.Linq;
  using System.Text;
  using System.Windows.Forms;

  using Multicad;
  using Multicad.AplicationServices;
  using Multicad.DatabaseServices;
  using Multicad.DatabaseServices.StandardObjects;
  using Multicad.Geometry;
  using Multicad.Runtime;
  using Multicad.Symbols;
  using Multicad.Symbols.Tables;

  public class SocketTableSample
  {
    // Создание шаблона для отчета
    private bool CreateTemplate(string fileName)
    {
      McTable table = new McTable();
      table.DefaultCell.HorizontalTextAlign = HorizTextAlign.Center;
      table.DefaultCell.VerticalTextAlign = VertTextAlign.Center;
      table.Columns.AddRange(0, 2);
      table.Columns[0].Width = 55;
      table.Columns[1].Width = 10;

      // Секция колонтитула первой страницы
      table.Rows.InsertSection(SectionTypeEnum.HeaderFirst, 0, 2);
      System.Drawing.Rectangle rect = new System.Drawing.Rectangle(0, 0, 1, 0);
      table.Merge(rect);
      table[0, 0].TextHeight = 3.5;
      table.Rows[0].Height = 10;
      table[0, 0].Value = "Ведомость электрических розеток по номеру помещения";

      foreach (var cell in table.Rows[0].Cells)
      {
        cell.SetBorderLineweight(BorderTypesEnum.ButBottom, -9);
      }

      table.Rows[1].Height = 8;
      foreach (var cell in table.Rows[1].Cells)
      {
        cell.TextHeight = 2.5;
      }

      table[1, 0].Value = "Артикул изделия";
      table[1, 1].Value = "#room";

      // Секция колонтитула
      table.Rows.InsertSection(SectionTypeEnum.Header, 2);
      table.Rows[3].Height = 8;
      foreach (var cell in table.Rows[3].Cells)
      {
        cell.TextHeight = 2.5;
      }

      table[3, 0].Value = "Артикул изделия";
      table[3, 1].Value = "#room";
      
      // Секция данных
      table.Rows.InsertSection(SectionTypeEnum.None, 4);
      table.Rows[5].Height = 6;
      table[5, 0].Value = "#code";
            
      if (!table.SaveToFile(fileName))
      {
        return false;
      }

      return true;
    }

    // Поиск ячейки в шаблоне, с которой нужно начинать заполнение
    private string findStartCell(McTable table, int startIndex, string keyword)
    {
      for (int i = startIndex; i < table.Rows.Count; i++)
      {
        foreach (var cell in table.Rows[i].Cells)
        {
          if (cell.Value == keyword)
          {
            return cell.AddressOfCell();
          }
        }
      }

      return null;
    }

    // Создание табличного отчета по выноскам
    [CommandMethod("smpl_CreateTableReport", CommandFlags.NoCheck | CommandFlags.NoPrefix)]
    public void smpl_CreateTableReport()
    {
      // Создание и загрузка шаблона таблицы и запуск редактора таблиц для возможного изменения шаблона
      const string FileName = "c:\\template.dat";
      McTable Table = new McTable();
      
      if (!CreateTemplate(FileName))
      {
        MessageBox.Show("Невозможно создать таблицу");
        return;
      }

      if (!Table.LoadFromFile(FileName))
      {
        MessageBox.Show("Невозможно загрузить таблицу");
        return;
      }

      // Предварительный просмотр таблицы, раскомментировать если нужен
      ////Table.OnEdit();
      
      // Выбор всех объектов типа "выноска для многослойных конструкций"
      McObjectId[] idSelecteds = ObjectFilter.Create(true).AddType(McNoteMultilayer.TypeID).GetObjects().ToArray();
      
      if (idSelecteds == null || idSelecteds.Length == 0)
      {
        MessageBox.Show("Объекты не выбраны");
        return;
      }
          
      // Структурирование данных выбранных объектов в виде словаря
      Dictionary<string, Dictionary<string, int>> slotsByName = new Dictionary<string, Dictionary<string, int>>();
      Dictionary<string, int> countByRoom = new Dictionary<string, int>();
      List<string> roomList = new List<string>();
      foreach (McObjectId id in idSelecteds)
      {
        McNoteMultilayer note = id.GetObject(); 
        if (slotsByName.ContainsKey(note.Units[1].Text))
        {
          countByRoom = slotsByName[note.Units[1].Text];
          if (countByRoom.ContainsKey(note.Units[2].Text))
          {
            countByRoom[note.Units[2].Text]++;
          }
          else
          {
            countByRoom[note.Units[2].Text] = 1;
          }

          slotsByName[note.Units[1].Text] = countByRoom;
        }
        else
        {
          countByRoom = new Dictionary<string, int>();
          slotsByName.Add(note.Units[1].Text, countByRoom);
          countByRoom.Add(note.Units[2].Text, 1);
        }

        if (!roomList.Exists(r => r == note.Units[2].Text))
        {
          roomList.Add(note.Units[2].Text);
        }

        roomList.Sort();
      }
      
      List<string> nameList = slotsByName.Keys.ToList();
      nameList.Sort();

      // Определение начальной позиции диапазона ячеек для хранения номеров помещений в колонтитулах 
      string roomCell_HeaderFirst = findStartCell(Table, 0, "#room");
      int roomColumnIndex = Table.GetCellOfAdress(roomCell_HeaderFirst).ColumnIndex;
      int roomRowIndex_HeaderFirst = Table.GetCellOfAdress(roomCell_HeaderFirst).RowIndex;

      string roomCell_Header = findStartCell(Table, roomRowIndex_HeaderFirst + 1, "#room");
      int roomRowIndex_Header = Table.GetCellOfAdress(roomCell_Header).RowIndex;

      // Определение начальной позиции диапазона ячек для хранения артикулов
      string productCodeCell = findStartCell(Table, 0, "#code");
      int productCodeColumnIndex = Table.GetCellOfAdress(productCodeCell).ColumnIndex;
      int productCodeRowIndex = Table.GetCellOfAdress(productCodeCell).RowIndex;

      // Заполнение колонтитулов
      for (int room = 0; room < roomList.Count; room++)
      {
        Table[roomRowIndex_HeaderFirst, room + roomColumnIndex].Value = roomList[room];
        Table[roomRowIndex_Header, room + roomColumnIndex].Value = roomList[room];
        if (room + 1 < roomList.Count)
        {
          Table.Columns.CopyRange(roomColumnIndex, room + roomColumnIndex + 1, 1, false);
        }
      }

      // Заполнение таблицы
      for (int name = 0; name < nameList.Count; name++)
      {
        Table[productCodeRowIndex + name, productCodeColumnIndex].Value = nameList[name];
        if (name + 1 < nameList.Count)
        {
          Table.Rows.CopyRange(productCodeRowIndex, productCodeRowIndex + name + 1, 1, false);
        }

        for (int room = 0; room < roomList.Count; room++)
        {
          if (slotsByName[nameList[name]].ContainsKey(roomList[room]))
          {
            Table[productCodeRowIndex + name, roomColumnIndex + room].Value = slotsByName[nameList[name]][roomList[room]].ToString();
          }
        }
      }

      // Индекс строки заголовка для колонтитула первой страницы
      int headerFirstRowIndex = -1;
      foreach (var row in Table.Rows)
      {
        if (row.SectionType == SectionTypeEnum.HeaderFirst)
        {
          headerFirstRowIndex = row.RowIndex;
          break;
        }
      }

      // Слияние ячеек в строке заголовка в колонтитуле первой страницы
      System.Drawing.Rectangle newRect = new System.Drawing.Rectangle(0, headerFirstRowIndex, Table.Columns.Count - 1, 0);
      Table.Merge(newRect);

      // Разбивка на страницы
      const double pageHeight = 50;
      double tableHeight = Table.DbEntity.BoundingBox.SizeByY;
      double tableWidth = Table.DbEntity.BoundingBox.SizeByX;
      if (tableHeight > pageHeight)
      {
        Table.Pages.SetPageHeight(pageHeight);
      }

      // Вставка страниц
      for (int n = 0; n < Table.Pages.Count; n++)
      {
        Table.Pages.SetOriginPage(n, new Point3d(n * (tableWidth + 5 * Table.DbEntity.Scale), 0, 0));
      }
    
      Table.PlaceObject(McEntity.PlaceFlags.Silent);
    }
  }
}
